///////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2019, ООО 1С-Софт
// Все права защищены. Эта программа и сопроводительные материалы предоставляются 
// в соответствии с условиями лицензии Attribution 4.0 International (CC BY 4.0)
// Текст лицензии доступен по ссылке:
// https://creativecommons.org/licenses/by/4.0/legalcode
///////////////////////////////////////////////////////////////////////////////////////////////////////

#Область СлужебныйПрограммныйИнтерфейс

// Обработчик события ПриОткрытии поля ввода формы для открытия формы ввода значения реквизита на разных языках.
//
// Параметры:
//  Объект - ДанныеФормыКоллекция - Объект на форме.
//  Элемент - ПолеФормы - Элемент формы, для которого будет открыта форма ввода на разных языках.
//  ИмяРеквизита - Строка - Имя реквизита, для которого будет открыта форма ввода на разных языках.
//  СтандартнаяОбработка - Булево - Признак выполнения стандартной (системной) обработки события.
//
Процедура ПриОткрытии(Объект, Элемент, ИмяРеквизита, СтандартнаяОбработка) Экспорт
	
	СтандартнаяОбработка = Ложь;
	
	ПараметрыФормы = Новый Структура;
	ПараметрыФормы.Вставить("Представления",   Объект.Представления);
	ПараметрыФормы.Вставить("Ссылка",          Объект.Ссылка);
	ПараметрыФормы.Вставить("ИмяРеквизита",    ИмяРеквизита);
	ПараметрыФормы.Вставить("ТекущиеЗначение", Элемент.ТекстРедактирования);
	ПараметрыФормы.Вставить("ТолькоПросмотр",  Элемент.ТолькоПросмотр);
	
	ДополнительныеПараметры = Новый Структура;
	ДополнительныеПараметры.Вставить("Объект", Объект);
	ДополнительныеПараметры.Вставить("ИмяРеквизита", ИмяРеквизита);
	
	Оповещение = Новый ОписаниеОповещения("ПослеВводаСтрокНаРазныхЯзыках", ЛокализацияКлиент, ДополнительныеПараметры);
	ОткрытьФорму("ОбщаяФорма.ВводНаРазныхЯзыках", ПараметрыФормы,,,,, Оповещение);
	
КонецПроцедуры

#КонецОбласти

#Область СлужебныеПроцедурыИФункции

Процедура ПослеВводаСтрокНаРазныхЯзыках(Результат, ДополнительныеПараметры) Экспорт
	
	Если ТипЗнч(Результат) <> Тип("Структура") Тогда
		Возврат;
	КонецЕсли;
	
	Объект = ДополнительныеПараметры.Объект;
	
	Для каждого Представление Из Результат.ЗначенияНаРазныхЯзыках Цикл
		Отбор = Новый Структура("КодЯзыка", Представление.КодЯзыка);
		НайденныеСтроки = Объект.Представления.НайтиСтроки(Отбор);
		Если НайденныеСтроки.Количество() > 0 Тогда
			Если ПустаяСтрока(Представление.ЗначениеРеквизита) 
				И СтрСравнить(Результат.ОсновнойЯзык, Представление.КодЯзыка) <> 0 Тогда
					Объект.Представления.Удалить(НайденныеСтроки[0]);
				Продолжить;
			КонецЕсли;
			СтрокаПредставлений = НайденныеСтроки[0];
		Иначе
			СтрокаПредставлений = Объект.Представления.Добавить();
			СтрокаПредставлений.КодЯзыка = Представление.КодЯзыка;
		КонецЕсли;
		СтрокаПредставлений[ДополнительныеПараметры.ИмяРеквизита] = Представление.ЗначениеРеквизита;
		
	КонецЦикла;
	
	Если Результат.Свойство("СтрокаНаТекущемЯзыке") Тогда
		Объект[ДополнительныеПараметры.ИмяРеквизита] = Результат.СтрокаНаТекущемЯзыке;
	КонецЕсли;
	
КонецПроцедуры

#КонецОбласти
